package cd.casic.module.machine.service.impl;
import cd.casic.module.machine.dto.MachineEnvDTO;
import cd.casic.module.machine.entity.MachineEnv;
import cd.casic.module.machine.exception.ServiceException;
import cd.casic.module.machine.mapper.MachineEnvMapper;
import cd.casic.module.machine.service.MachineEnvService;
import cd.casic.module.machine.utils.PageResult;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import jakarta.annotation.Resource;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 环境变量服务实现类
 */
@Service
public class MachineEnvServiceImpl extends ServiceImpl<MachineEnvMapper, MachineEnv> implements MachineEnvService {

    @Resource
    private MachineEnvMapper machineEnvMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean add(MachineEnvDTO machineEnvDTO) {
        // 参数校验
        if (machineEnvDTO==null) {
            throw new ServiceException(ServiceException.MACHINE_ENV_NULL,"环境变量不能为空");
        }

        // 检查键是否合法
        if (!isValidKey(machineEnvDTO.getEnvKey())) {
            throw  new ServiceException(ServiceException.MACHINE_ENV_KEY_ILLEGAL,"环境变量键不合法");
        }

        // 判断是否敏感变量
        boolean isSensitive = isSensitive(machineEnvDTO.getEnvKey());

        MachineEnv machineEnv = new MachineEnv();
        BeanUtils.copyProperties(machineEnvDTO, machineEnv);
        return save(machineEnv);
    }

    @Override
    public void deleteByMachineId(Long machineId) {
        this.removeById(machineId);
    }



    @Override
    public MachineEnvDTO getByMachineId(Long machineId) {
        if (machineId == null) {
            return null;
        }

        MachineEnv machineEnv = getOne(
                new LambdaQueryWrapper<MachineEnv>()
                        .eq(MachineEnv::getMachineId, machineId)
        );

        return machineEnv != null ? convertToDTO(machineEnv) : null;

    }


    @Override
    public PageResult<MachineEnvDTO> listEnv(MachineEnvDTO machineEnvDTO) {

        // 构建查询条件
        LambdaQueryWrapper<MachineEnv> queryWrapper = new LambdaQueryWrapper<>();

        // 环境变量键模糊查询
        if (!StringUtils.isEmpty(machineEnvDTO.getEnvKey())) {
            queryWrapper.like(MachineEnv::getEnvKey, machineEnvDTO.getEnvKey());
        }

        // 机器ID模糊查询
        if (!StringUtils.isEmpty(machineEnvDTO.getMachineId())) {
            queryWrapper.like(MachineEnv::getMachineId, machineEnvDTO.getMachineId());
        }

        // 是否敏感
        if (!StringUtils.isEmpty(machineEnvDTO.getSensitive())) {
            queryWrapper.eq(MachineEnv::getSensitive, machineEnvDTO.getSensitive());
        }

        // 创建时间范围查询
        if (!StringUtils.isEmpty(machineEnvDTO.getCreateDate())) {
            queryWrapper.ge(MachineEnv::getCreateDate, machineEnvDTO.getCreateDate());
        }

        // 排序
        if (!StringUtils.isEmpty(machineEnvDTO.getSortField())) {
            boolean isAsc = "asc".equalsIgnoreCase(machineEnvDTO.getSortDirection());
            switch (machineEnvDTO.getSortField()) {
                case "envKey":
                    queryWrapper.orderBy(true, isAsc, MachineEnv::getEnvKey);
                    break;
                case "machineId":
                    queryWrapper.orderBy(true, isAsc, MachineEnv::getMachineId);
                    break;
                case "createTime":
                default:
                    queryWrapper.orderBy(true, isAsc, MachineEnv::getCreateDate);
                    break;
            }
        }

        // 分页查询
        Page<MachineEnv> page = machineEnvMapper.selectPage(new Page<>(machineEnvDTO.getPageIndex(), machineEnvDTO.getPageSize()), queryWrapper);

        // 转换结果
        List<MachineEnvDTO> dtoList = page.getRecords().stream()
                .map(this::convertToDTO)
                .collect(Collectors.toList());

        // 构建分页结果
        return PageResult.<MachineEnvDTO>builder()
                .pageNum(page.getCurrent())
                .pageSize(page.getSize())
                .total(page.getTotal())
                .pages(page.getPages())
                .list(dtoList)
                .build();
    }

    @Override
    public void deleteList(List<Long> ids) {
        this.machineEnvMapper.deleteBatchIds(ids);
    }

    @Override
    public boolean update(MachineEnvDTO machineEnvDTO) {
        MachineEnv machineEnv = new MachineEnv();
        BeanUtils.copyProperties(machineEnvDTO,machineEnv);
        return this.updateById(machineEnv);
    }

    // 转换实体为DTO
    private MachineEnvDTO convertToDTO(MachineEnv machineEnv) {
        MachineEnvDTO dto = new MachineEnvDTO();
        dto.setId(machineEnv.getId());
        dto.setEnvKey(machineEnv.getEnvKey());
        dto.setEnvValue(machineEnv.getEnvValue());
        dto.setSensitive(machineEnv.getSensitive());
        dto.setDescription(machineEnv.getDescription());
        dto.setCreateDate(machineEnv.getCreateDate());
        dto.setUpdateDate(machineEnv.getUpdateDate());
        return dto;
    }

    // 检查环境变量键是否合法
    private boolean isValidKey(String key) {
        return key.matches("^[a-zA-Z_][a-zA-Z0-9_]*$");
    }

    // 判断是否为敏感变量
    private boolean isSensitive(String key) {
        if (key == null) {
            return false;
        }
        String upperKey = key.toUpperCase();
        return upperKey.contains("PASSWORD") || upperKey.contains("SECRET") ||
                upperKey.contains("TOKEN") || upperKey.contains("KEY");
    }
}
