package cd.casic.module.machine.service.impl;
import cd.casic.module.machine.dto.MachineProxyDTO;
import cd.casic.module.machine.entity.MachineProxy;
import cd.casic.module.machine.enums.MachineProxyStatus;
import cd.casic.module.machine.enums.MachineProxyType;
import cd.casic.module.machine.exception.ServiceException;
import cd.casic.module.machine.mapper.MachineProxyMapper;
import cd.casic.module.machine.service.MachineProxyService;
import cd.casic.module.machine.utils.EnumUtils;
import cd.casic.module.machine.utils.PageResult;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import jakarta.annotation.Resource;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 机器代理服务实现类
 */
@Service
public class MachineProxyServiceImpl extends ServiceImpl<MachineProxyMapper, MachineProxy> implements MachineProxyService {
    @Resource
    private MachineProxyMapper machineProxyMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean register(MachineProxyDTO machineProxyDTO) {
        // 创建代理记录
        MachineProxy proxy = new MachineProxy();
        BeanUtils.copyProperties(machineProxyDTO, proxy);
        proxy.setProxyTypeCode(EnumUtils.getEnumByMessage(machineProxyDTO.getProxyType(), MachineProxyType.class).getCode());
        proxy.setVersion("1.0.0");
        proxy.setStatusCode(MachineProxyStatus.INSTALLING.getCode());
        return save(proxy);
    }

    @Override
    public boolean updateStatus(MachineProxyDTO machineProxyDTO) {
        // 参数校验
        if (machineProxyDTO == null) {
            throw new ServiceException(ServiceException.MACHINE_PROXY_DTO_NULL,"MachineProxyDTO对象为空");
        }

        // 查询代理
        MachineProxy proxy = this.getById(machineProxyDTO.getId());

        if (proxy == null) {
            throw new ServiceException(ServiceException.MACHINE_PROXY_NULL,"代理不存在");
        }

        // 更新状态
        proxy.setStatusCode(EnumUtils.getEnumByMessage(machineProxyDTO.getStatus(),MachineProxyStatus.class).getCode());
        proxy.setUpdateDate(new Date());
        return updateById(proxy);
    }



    @Override
    public Map<String, Long> getStatusStatistics() {
        List<MachineProxy> proxyList = list();

        if (CollectionUtils.isEmpty(proxyList)) {
            return Collections.emptyMap();
        }
        return proxyList.stream()
                .map(proxy -> EnumUtils.getEnumByCode(proxy.getStatusCode(), MachineProxyStatus.class).getMessage())
                .collect(Collectors.groupingBy(
                        Function.identity(),
                        Collectors.counting() // 统计每个分组的元素数量
                ));
    }

    @Override
    public void updateConfig(MachineProxyDTO machineProxyDTO) {
        // 参数校验
        if (machineProxyDTO == null) {
            throw new ServiceException(ServiceException.MACHINE_PROXY_DTO_NULL,"MachineProxyDTO对象为空");
        }

        // 查询代理
        MachineProxy proxy = getById(machineProxyDTO.getId());

        if (proxy == null) {
            throw new ServiceException(ServiceException.MACHINE_PROXY_NULL,"代理不存在");
        }
        // 更新配置
        proxy.setConfig(machineProxyDTO.getConfig());
        proxy.setUpdateDate(new Date());
        updateById(proxy);
    }



    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(List<Long> ids) {
        // 参数校验
        if (CollectionUtils.isEmpty(ids)) {
            throw new ServiceException(ServiceException.PARAMETER_ERROR,"参数错误");
        }

        // 查询在线代理
        List<MachineProxy> onlineProxies = list(new LambdaQueryWrapper<MachineProxy>()
                .in(MachineProxy::getId, ids)
                .eq(MachineProxy::getStatusCode, MachineProxyStatus.ONLINE.getCode()));

        if (!CollectionUtils.isEmpty(onlineProxies)) {
            List<Long> onlineIds = onlineProxies.stream()
                    .map(MachineProxy::getId)
                    .collect(Collectors.toList());
            throw new IllegalArgumentException("以下代理处于在线状态，无法删除: " + String.join( ",", (CharSequence) onlineIds));
        }

        // 批量逻辑删除
        remove(new LambdaQueryWrapper<MachineProxy>()
                .in(MachineProxy::getId, ids));
    }

    @Override
    public PageResult<MachineProxyDTO> list(MachineProxyDTO machineProxyDTO) {
        QueryWrapper<MachineProxy> queryWrapper = getMachineProxyQueryWrapper(machineProxyDTO);
        Page<MachineProxy> page = machineProxyMapper.selectPage(new Page<>(machineProxyDTO.getPageIndex(), machineProxyDTO.getPageSize()), queryWrapper);
        List<MachineProxyDTO> machineProxyDtos = page.getRecords().stream().map(machineProxy -> {
            MachineProxyDTO dto = new MachineProxyDTO();
            BeanUtils.copyProperties(machineProxy, dto);
            dto.setProxyType(EnumUtils.getEnumByCode(machineProxy.getStatusCode(), MachineProxyType.class).getMessage());
            dto.setStatus(EnumUtils.getEnumByCode(machineProxy.getStatusCode(), MachineProxyStatus.class).getMessage());
            return dto;
        }).toList();
        return new PageResult<>(
                page.getCurrent(),
                page.getSize(),
                page.getTotal(),
                page.getPages(),
                machineProxyDtos
        );
    }

    private QueryWrapper<MachineProxy> getMachineProxyQueryWrapper(MachineProxyDTO machineProxyDTO){
        QueryWrapper<MachineProxy> queryWrapper = new QueryWrapper<>();
        if (machineProxyDTO.getHostIp() != null && !machineProxyDTO.getHostIp().isEmpty()) {
            queryWrapper.eq("host_ip", machineProxyDTO.getHostIp());
        }
        if (machineProxyDTO.getSshPort() != null && !machineProxyDTO.getSshPort().isEmpty()) {
            queryWrapper.eq("ssh_port", machineProxyDTO.getSshPort());
        }
        if (machineProxyDTO.getUsername() != null && !machineProxyDTO.getUsername().isEmpty()) {
            queryWrapper.eq("username", machineProxyDTO.getUsername());
        }
        if (machineProxyDTO.getDescription() != null && !machineProxyDTO.getDescription().isEmpty()) {
            queryWrapper.eq("description", machineProxyDTO.getDescription());
        }
        if (machineProxyDTO.getStatus() != null && !machineProxyDTO.getStatus().isEmpty()) {
            queryWrapper.eq("status_code", EnumUtils.getEnumByMessage(machineProxyDTO.getStatus(),MachineProxyStatus.class).getCode());
        }
        return queryWrapper;
    }

}
