package cd.casic.module.machine.service.impl;

import cd.casic.module.machine.handler.ConnectionSession;
import cd.casic.module.machine.mapper.MachineInfoMapper;
import cd.casic.module.machine.dto.MachineInfoDto;
import cd.casic.module.machine.entity.MachineInfo;
import cd.casic.module.machine.enums.AuthenticationType;
import cd.casic.module.machine.enums.ConnectionStatus;
import cd.casic.module.machine.enums.MachineInfoStatus;
import cd.casic.module.machine.exception.ServiceException;
import cd.casic.module.machine.service.MachineInfoService;
import cd.casic.module.machine.utils.EnumUtils;
import cd.casic.module.machine.utils.PageResult;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

@Slf4j
@Service
public class MachineinfoServiceImpl extends ServiceImpl<MachineInfoMapper, MachineInfo> implements MachineInfoService {

    int ENABLE=1;
    int UN_ENABLE=0;

    @Resource
    private MachineInfoMapper machineInfoMapper;

    /**
     * 会话ID生成器
     */
    private final AtomicInteger sessionIdGenerator = new AtomicInteger(1000);

    /**
     * 会话管理：会话ID -> 连接会话
     */
    private final Map<String, ConnectionSession> sessions = new ConcurrentHashMap<>();

    /**
     * 机器名称 -> 会话ID
     */
    private final Map<String, String> machineSessionMapping = new ConcurrentHashMap<>();
    @Override
    public boolean addMachineInfo(MachineInfoDto machineInfoDto) {
        if (machineInfoDto == null) {
            throw new ServiceException(ServiceException.MACHINE_INFO_NULL, "机器信息为空");
        }
        MachineInfo machineInfo = new MachineInfo();
        BeanUtils.copyProperties(machineInfoDto, machineInfo);
        machineInfo.setStatusCode(1);
        machineInfo.setAuthenticationTypeCode(
            "密码认证".equals(machineInfoDto.getAuthenticationType())
                ? AuthenticationType.PASSWORD.getCode()
                : AuthenticationType.SECRET_KEY.getCode()
        );

        return this.save(machineInfo);
    }

    @Override
    public PageResult<MachineInfoDto> listMachineInfo(MachineInfoDto machineInfoDto) {
        QueryWrapper<MachineInfo> queryWrapper = getMachineInfoQueryWrapper(machineInfoDto);
        Page<MachineInfo> page = machineInfoMapper.selectPage(
            new Page<>(machineInfoDto.getPageIndex(), machineInfoDto.getPageSize()),
            queryWrapper
        );

        List<MachineInfoDto> machineInfoDtos = page.getRecords().stream()
            .map(machineInfo -> {
                MachineInfoDto dto = new MachineInfoDto();
                BeanUtils.copyProperties(machineInfo, dto);
                // 直接调用原有枚举转换方法
                dto.setStatus(EnumUtils.getEnumByCode(machineInfo.getStatus(), MachineInfoStatus.class).getMessage());
                dto.setAuthenticationType(EnumUtils.getEnumByCode(machineInfo.getAuthenticationType(), AuthenticationType.class).getMessage());
                return dto;
            })
            .toList();

        return new PageResult<>(
            page.getCurrent(),
            page.getSize(),
            page.getTotal(),
            page.getPages(),
            machineInfoDtos
        );
    }

    @Override
    public boolean updateMachineInfo(MachineInfoDto machineInfoDto) {
        MachineInfo machineInfo = new MachineInfo();
        BeanUtils.copyProperties(machineInfoDto, machineInfo);
        machineInfo.setAuthenticationType(
            "密码认证".equals(machineInfoDto.getAuthenticationType())
                ? AuthenticationType.PASSWORD
                : AuthenticationType.SECRET_KEY
        );
        return this.updateById(machineInfo);
    }

    @Override
    public boolean updateStatus(Long machineInfoId, String status) {
        UpdateWrapper<MachineInfo> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", machineInfoId).set("status", status);
        return this.update(updateWrapper);
    }

    @Override
    public boolean bindingSecretKey(Long machineInfoId, Long secretKeyId) {
        UpdateWrapper<MachineInfo> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", machineInfoId).set("secretKeyId", secretKeyId);
        return this.update(updateWrapper);
    }

    @Override
    public void deleteList(List<Long> machineInfoIds) {
        machineInfoMapper.selectBatchIds(machineInfoIds).forEach(machineInfo -> {
            if (machineInfo.getStatusCode() == 1){
                this.removeById(machineInfo.getId());
            }
        });
    }

    @Override
    public void deleteMachineInfo(Long machineInfoId) {
        MachineInfo machineInfo = this.getById(machineInfoId);
        if (machineInfo.getStatusCode() == 1){
            this.removeById(machineInfoId);
        }
    }

    private QueryWrapper<MachineInfo> getMachineInfoQueryWrapper(MachineInfoDto machineInfoDto) {
        QueryWrapper<MachineInfo> queryWrapper = new QueryWrapper<>();
        if (machineInfoDto.getStatus() != null && !machineInfoDto.getStatus().isEmpty()){
            queryWrapper.eq("status_code", EnumUtils.getEnumByMessage(machineInfoDto.getStatus(), MachineInfoStatus.class).getCode());
        }
        if (machineInfoDto.getName() != null && !machineInfoDto.getName().isEmpty()) {
            queryWrapper.like("name", machineInfoDto.getName());
        }
        if (machineInfoDto.getTag() != null && !machineInfoDto.getTag().isEmpty()) {
            queryWrapper.like("tag", machineInfoDto.getTag());
        }
        if (machineInfoDto.getHostIp() != null && !machineInfoDto.getHostIp().isEmpty()) {
            queryWrapper.like("host_ip", machineInfoDto.getHostIp());
        }
        if (machineInfoDto.getDescription() != null && !machineInfoDto.getDescription().isEmpty()) {
            queryWrapper.like("description", machineInfoDto.getDescription());
        }
        return queryWrapper;
    }




    @Override
    public boolean testConnection(MachineInfo machineInfo) {
        if (machineInfo.getStatus().getCode()==UN_ENABLE) {
            throw new RuntimeException("机器不可用");
        }
        log.info("测试机器连接: {}", machineInfo.getHostIp());

        try (ConnectionSession session = createSession(machineInfo)) {
            session.connect();
            return true;
        } catch (Exception e) {
            log.error("机器连接测试失败: {}", e.getMessage(), e);
            return false;
        }
    }

    @Override
    public ConnectionStatus getConnectionStatus(String machineName) {
        String sessionId = machineSessionMapping.get(machineName);
        if (sessionId == null) {
            return ConnectionStatus.DISCONNECTED;
        }

        ConnectionSession session = sessions.get(sessionId);
        return session != null ? session.getStatus() : ConnectionStatus.DISCONNECTED;
    }

    @Override
    public Map<String, ConnectionStatus> getAllConnectionStatus() {
        Map<String, ConnectionStatus> result = new HashMap<>();

        machineSessionMapping.forEach((machineName, sessionId) -> {
            ConnectionSession session = sessions.get(sessionId);
            result.put(machineName, session != null ? session.getStatus() : ConnectionStatus.DISCONNECTED);
        });

        return result;
    }

    @Override
    public String connect(MachineInfo machineInfo) {
        if (machineInfo.getStatus().getCode()==UN_ENABLE) {
            throw new RuntimeException("机器不可用");
        }
        log.info("建立机器连接: {}", machineInfo.getHostIp());

        // 检查是否已连接
        String existingSessionId = machineSessionMapping.get(machineInfo.getName());
        if (existingSessionId != null) {
            ConnectionSession existingSession = sessions.get(existingSessionId);
            if (existingSession != null && existingSession.getStatus() == ConnectionStatus.CONNECTED) {
                log.info("机器已连接，返回现有会话: {}", machineInfo.getHostIp());
                return existingSessionId;
            }
        }

        try {
            ConnectionSession session = createSession(machineInfo);
            session.connect();

            // 生成会话ID
            String sessionId = generateSessionId();

            // 保存会话
            sessions.put(sessionId, session);
            machineSessionMapping.put(machineInfo.getName(), sessionId);

            log.info("机器连接成功: {}, 会话ID: {}", machineInfo.getHostIp(), sessionId);
            return sessionId;
        } catch (Exception e) {
            log.error("机器连接失败: {}", e.getMessage(), e);
            throw new RuntimeException("机器连接失败: " + e.getMessage(), e);
        }
    }

    @Override
    public boolean disconnect(String sessionId) {
        log.info("断开机器连接: {}", sessionId);

        ConnectionSession session = sessions.get(sessionId);
        if (session == null) {
            log.warn("会话不存在: {}", sessionId);
            return false;
        }

        try {
            session.disconnect();

            // 清理会话
            sessions.remove(sessionId);
            machineSessionMapping.entrySet().removeIf(entry -> entry.getValue().equals(sessionId));

            log.info("机器连接已断开: {}", sessionId);
            return true;
        } catch (Exception e) {
            log.error("断开连接失败: {}", e.getMessage(), e);
            return false;
        }
    }

    @Override
    public String executeCommand(String sessionId, String command) {
        log.info("执行命令: {}, 会话ID: {}", command, sessionId);

        ConnectionSession session = sessions.get(sessionId);
        if (session == null) {
            throw new RuntimeException("会话不存在: " + sessionId);
        }

        if (session.getStatus() != ConnectionStatus.CONNECTED) {
            throw new RuntimeException("会话未连接: " + sessionId);
        }

        try {
            return session.executeCommand(command);
        } catch (Exception e) {
            log.error("命令执行失败: {}", e.getMessage(), e);
            throw new RuntimeException("命令执行失败: " + e.getMessage(), e);
        }
    }

    @Override
    public boolean uploadFile(String sessionId, String localFilePath, String remoteFilePath) {
        log.info("上传文件: {} -> {}, 会话ID: {}", localFilePath, remoteFilePath, sessionId);

        ConnectionSession session = sessions.get(sessionId);
        if (session == null) {
            throw new RuntimeException("会话不存在: " + sessionId);
        }

        if (session.getStatus() != ConnectionStatus.CONNECTED) {
            throw new RuntimeException("会话未连接: " + sessionId);
        }

        try {
            return session.uploadFile(localFilePath, remoteFilePath);
        } catch (Exception e) {
            log.error("文件上传失败: {}", e.getMessage(), e);
            throw new RuntimeException("文件上传失败: " + e.getMessage(), e);
        }
    }

    @Override
    public boolean downloadFile(String sessionId, String remoteFilePath, String localFilePath) {
        log.info("下载文件: {} -> {}, 会话ID: {}", remoteFilePath, localFilePath, sessionId);

        ConnectionSession session = sessions.get(sessionId);
        if (session == null) {
            throw new RuntimeException("会话不存在: " + sessionId);
        }

        if (session.getStatus() != ConnectionStatus.CONNECTED) {
            throw new RuntimeException("会话未连接: " + sessionId);
        }

        try {
            return session.downloadFile(remoteFilePath, localFilePath);
        } catch (Exception e) {
            log.error("文件下载失败: {}", e.getMessage(), e);
            throw new RuntimeException("文件下载失败: " + e.getMessage(), e);
        }
    }

    /**
     * 创建连接会话
     */
    private ConnectionSession createSession(MachineInfo machineInfo) {
        return new ConnectionSession(machineInfo);
    }

    /**
     * 生成会话ID
     */
    private String generateSessionId() {
        return "session-" + sessionIdGenerator.incrementAndGet();
    }
}
