package cd.casic.module.machine.utils;

import cd.casic.module.machine.configuration.AliYunConfig;
import cd.casic.module.machine.exception.ServiceException;
import com.aliyun.oss.OSS;
import com.aliyun.oss.model.OSSObject;
import jakarta.annotation.Resource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;


@Component
public class AliOssUtil{

//    @Autowired
    private OSS ossClient;

//    @Resource
    private AliYunConfig aliyunConfig;

    private static final Logger logger = LoggerFactory.getLogger(AliOssUtil.class);

    public String save(MultipartFile file) {
        try {
            String fileName = generateUniqueFileName(file.getName());
            ossClient.putObject(
                    // 存储桶名称
                    aliyunConfig.getBucketName(),
                    //对象键（Object Key），即文件在 OSS 中的完整路径和名称
                    fileName,
                    //文件内容的输入流，用于读取待上传的文件数据。
                    file.getInputStream(),
                    //文件的元数据信息，如内容类型（Content-Type）、缓存策略、文件大小等。
                    null
            );
            return fileName;
        } catch (Exception e) {
            logger.info("文件上传失败:{}", e.getMessage());
            throw new ServiceException(ServiceException.UPLOADING_FILE_FAIL, "上传文件失败:" + e.getMessage());
        }
    }

    public ResponseEntity<InputStreamResource> downloadFile(String fileName) {
        try {
            // 读取文件流
            InputStream inputStream = read(fileName);

            // 处理文件名编码（防止中文乱码）
            String encodedFileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8)
                    .replaceAll("\\+", "%20");

            // 设置响应头
            HttpHeaders headers = new HttpHeaders();
            headers.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=" + encodedFileName);
            headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
            headers.setCacheControl("no-cache, no-store, must-revalidate");

            return ResponseEntity.ok()
                    .headers(headers)
                    .body(new InputStreamResource(inputStream));
        } catch (Exception e) {
            logger.error("下载失败:{}", e.getMessage());
            throw new ServiceException(ServiceException.DOWNLOAD_FILE_FAIL, "下载失败: " + e.getMessage());
        }
    }

    public void deleteFile(String fileName) {
        try {
            if (fileName == null || fileName.isEmpty()) {
                throw new ServiceException(ServiceException.FILENAME_NULL, "文件名不能为空");
            }

            // 调用 OSS 客户端删除文件
            ossClient.deleteObject(aliyunConfig.getBucketName(), fileName);
            logger.info("文件删除成功: {}", fileName);
        } catch (Exception e) {
            logger.error("删除文件失败: {}", e.getMessage());
            throw new ServiceException(ServiceException.DELETE_FILE_FAIL, "删除文件失败: " + e.getMessage());
        }
    }

    public InputStream read(String fileName) {
        try {
            if (fileName == null || fileName.isEmpty()) {
                throw new ServiceException(ServiceException.FILENAME_NULL, "文件名不能为空");
            }
            OSSObject ossObject = ossClient.getObject(aliyunConfig.getBucketName(), fileName);
            return ossObject.getObjectContent();
        } catch (Exception e) {
            logger.error("读取文件失败:{}", e.getMessage());
            throw new ServiceException(ServiceException.READ_FILE_FAIL, "读取文件失败: " + e.getMessage());
        }
    }

    // 生成带时间戳的唯一文件名
    private String generateUniqueFileName(String originalFileName) {
        String timestamp = String.valueOf(System.currentTimeMillis());
        String ext = originalFileName.substring(originalFileName.lastIndexOf("."));
        return timestamp + ext;
    }

}
