package cd.casic.module.process.pipeline.execute.service.impl;


import cd.casic.ci.commons.bean.engine.execute.PipelineDetails;
import cd.casic.ci.commons.bean.engine.execute.PipelineRunMsg;
import cd.casic.ci.commons.bean.engine.postprocess.Postprocess;
import cd.casic.ci.commons.bean.exception.SystemException;
import cd.casic.ci.commons.bean.process.definition.Pipeline;
import cd.casic.ci.commons.bean.process.definition.Variable;
import cd.casic.ci.commons.bean.process.instance.PipelineInstance;
import cd.casic.ci.commons.bean.process.instance.PipelineInstanceQuery;
import cd.casic.ci.commons.bean.process.setting.Scm;
import cd.casic.ci.commons.bean.process.stage.Stage;
import cd.casic.ci.commons.bean.work.agent.AgentMessage;
import cd.casic.ci.commons.bean.work.agent.model.Agent;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import cd.casic.module.process.engine.task.TasksExecService;
import cd.casic.module.process.pipeline.definition.PipelineService;
import cd.casic.module.process.pipeline.execute.service.PipelineExecService;
import cd.casic.module.process.pipeline.instance.service.PipelineInstanceServiceImpl;
import cd.casic.module.process.support.variable.service.VariableService;
import cd.casic.module.process.process.instance.PipelineInstanceService;
import cd.casic.module.process.setting.service.ResourcesService;
import cd.casic.module.process.setting.service.ScmService;
import cd.casic.module.process.stages.service.StageExecService;
import cd.casic.module.process.stages.service.StageService;
import cd.casic.module.process.support.agent.service.AgentService;
import cd.casic.module.process.support.disk.service.DiskService;
import cd.casic.module.process.support.version.service.PipelineVersionService;
import cd.casic.module.process.support.postprocess.service.PostprocessExecService;
import cd.casic.module.process.support.util.PipelineUtilService;
import cd.casic.module.process.support.util.util.PipelineFinal;
import cd.casic.module.process.toolkit.join.JoinTemplate;
import cd.casic.module.process.ws.server.SocketServerHandler;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.socket.WebSocketSession;
//import org.springframework.web.socket.WebSocketSession;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 流水线运行服务
 */
@Service
//@Exporter
public class PipelineExecServiceImpl implements PipelineExecService {

    @Autowired
    PipelineService pipelineService;

    @Autowired
    PostprocessExecService postExecService;

    @Autowired
    PipelineInstanceService pipelineInstanceService;

    @Autowired
    TasksExecService tasksExecService;

    @Autowired
    StageExecService stageExecService;

    @Autowired
    ResourcesService resourcesService;

    @Autowired
    PipelineVersionService versionService;

    @Autowired
    PipelineUtilService utilService;

    @Autowired
    DiskService diskService;

    @Autowired
    StageService stageService;

    @Autowired
    JoinTemplate joinTemplate;

    @Autowired
    ScmService scmService;

    @Autowired
    VariableService variableService;

    @Autowired
    AgentService agentService;

    public final Logger logger = LoggerFactory.getLogger(PipelineExecServiceImpl.class);

    //流水线id:流水线实例id
    public static final Map<String, String> pipelineIdOrInstanceId = new HashMap<>();

    //流水线id:agent
    public static final Map<String, Agent> pipelineIdOrAgentId = new HashMap<>();


    /**
     * 流水线开始运行
     *
     * @param runMsg 流水线id
     * @return 是否正在运行
     */
    @Override
    public PipelineInstance start(PipelineRunMsg runMsg) {
        Agent agent;
        if (StringUtils.isEmpty(runMsg.getAgentId())) {
            agent = agentService.findDefaultAgent();
        } else {
            agent = agentService.findAgent(runMsg.getAgentId());
        }
        if (Objects.isNull(agent)) {
            throw new ServiceException(GlobalErrorCodeConstants.LOCKED.getCode(), "无法获取到流水线执行Agent！");
        }

        WebSocketSession session = SocketServerHandler.sessionMap.get(agent.getAddress());
        if (Objects.isNull(session)) {
            throw new ServiceException(GlobalErrorCodeConstants.LOCKED.getCode(), "流水线Agent断开连接，无法执行。");
        }

        // 判断同一任务是否在运行
        Pipeline pipeline = validExecPipeline(runMsg);
        String pipelineId = pipeline.getId();
        pipelineIdOrAgentId.put(pipelineId, agent);

        // 判断磁盘空间是否足够
        diskService.validationStorageSpace();

        // 资源限制
        resourcesService.judgeResources();

        // 进入执行
        runMsg.setPipeline(pipeline);
        runMsg.setAgent(agent);
        return beginExecPipeline(runMsg);
    }

    /**
     * 放入正在执行的流水线缓存中
     *
     * @param runMsg 流水线id
     * @return 流水线信息
     */
    public Pipeline validExecPipeline(PipelineRunMsg runMsg) {
        String pipelineId = runMsg.getPipelineId();

        List<Stage> allMainStage = stageService.findAllMainStage(pipelineId);
        if (allMainStage.isEmpty()) {
            throw new ServiceException(2000, "当前流水线不存在可构建任务！");
        }

        Boolean isVip = versionService.isVip();

        Pipeline pipeline = pipelineService.findPipelineById(pipelineId);

        int size = pipelineIdOrInstanceId.size();

        // 资源限制放入缓存中等待执行
        if ((!isVip && size >= 2) || (isVip && size >= 4)) {
            throw new ServiceException(2000, "并行任务已满，等待执行！");
        }
        return pipeline;
    }

    /**
     * 执行流水线
     *
     * @param runMsg 流水线信息
     * @return 流水线实例
     */
    public PipelineInstance beginExecPipeline(PipelineRunMsg runMsg) {
        String pipelineId = runMsg.getPipelineId();
        Pipeline pipeline = pipelineService.findPipelineById(pipelineId);
        pipeline.setState(2);
        pipelineService.updatePipeline(pipeline);
        runMsg.setPipeline(pipeline);

        logger.info("流水线{}开始运行", pipeline.getName());
        PipelineInstance pipelineInstance = pipelineInstanceService.initializeInstance(runMsg);
        // 添加到缓存
        String instanceId = pipelineInstance.getInstanceId();
        pipelineInstanceService.instanceRuntime(pipelineInstance.getInstanceId());
        joinTemplate.joinQuery(pipelineInstance);

        // 运行实例放入内存中
        pipelineIdOrInstanceId.put(pipelineId, instanceId);

        try {
            // 创建多阶段运行实例
            List<Stage> stageList = stageExecService.createStageExecInstance(pipelineId, instanceId);

            List<Postprocess> postprocessList = postExecService.createPipelinePostInstance(pipelineId, instanceId);

            PipelineDetails pipelineDetails = new PipelineDetails();

            // 流水线基本运行信息
            pipelineDetails.setPipelineId(pipelineId);
            pipelineDetails.setInstanceId(instanceId);
            pipelineDetails.setRunWay(runMsg.getRunWay());
            pipelineDetails.setAgent(runMsg.getAgent());

            // 流水线运行任务
            pipelineDetails.setStageList(stageList);

            // 流水线后置处理
            pipelineDetails.setPostprocessList(postprocessList);

            // 数据路径，源码，日志保存
            String sourceDir = utilService.findPipelineDefaultAddress(pipelineId, 1);
            String logDir = utilService.findPipelineDefaultAddress(pipelineId, 2);
            pipelineDetails.setSourceDir(sourceDir);
            pipelineDetails.setLogDir(logDir);

            // 环境
            List<Scm> scmList = scmService.findAllPipelineScm();
            pipelineDetails.setScmList(scmList);

            // 变量
            List<Variable> variableList = variableService.findAllVariable(pipelineId);
            pipelineDetails.setVariableList(variableList);

            AgentMessage agentMessage = new AgentMessage();
            agentMessage.setType("exec");
            agentMessage.setMessage(pipelineDetails);
            agentMessage.setPipelineId(pipelineId);

            Agent agent = pipelineDetails.getAgent();

            String id = agent.getAddress();

            WebSocketSession session = SocketServerHandler.sessionMap.get(id);
            if (Objects.isNull(session)) {
                throw new SystemException("客户端推送消息失败，无法获取客户端连接,客户端信息：" + id);
            }

            try {
                SocketServerHandler.instance().sendHandleMessage(id, agentMessage);
            } catch (Exception e) {
                throw new SystemException("客户端推送消息失败,错误信息：" + e.getMessage());
            }
        } catch (Exception e) {
            logger.error("流水线执行出错了：{}", e.getMessage());
            stop(pipelineId);
        }
        return pipelineInstance;
    }

    @Override
    public void stop(String pipelineId) {
        Pipeline pipeline = pipelineService.findPipelineById(pipelineId);

        Agent agent = pipelineIdOrAgentId.get(pipelineId);
        if (Objects.isNull(agent)) {
            pipeline.setState(1);
            pipelineService.updatePipeline(pipeline);

            PipelineInstanceQuery pipelineInstanceQuery = new PipelineInstanceQuery();
            pipelineInstanceQuery.setState(PipelineFinal.RUN_RUN);
            pipelineInstanceQuery.setPipelineId(pipelineId);
            List<PipelineInstance> pipelineInstanceList = pipelineInstanceService.findPipelineInstanceList(pipelineInstanceQuery);
            for (PipelineInstance pipelineInstance : pipelineInstanceList) {
                String instanceId = pipelineInstance.getInstanceId();
                pipelineInstance.setRunStatus(PipelineFinal.RUN_HALT);
                int runtime = pipelineInstanceService.findInstanceRuntime(instanceId);
                pipelineInstance.setRunTime(runtime);
                pipelineInstanceService.updateInstance(pipelineInstance);
            }
            removeExecCache(pipelineId);
            return;
        }

        try {
            AgentMessage agentMessage = new AgentMessage();
            agentMessage.setType("stop");
            agentMessage.setMessage(pipelineId);
            agentMessage.setPipelineId(pipelineId);
            SocketServerHandler.instance().sendHandleMessage(agent.getAddress(), agentMessage);
        } catch (Exception e) {
            logger.error(e.getMessage());
        }

        // 添加资源配置
        // resourcesService.instanceResources(integer);

        // 更新流水线状态
        pipeline.setState(1);
        pipelineService.updatePipeline(pipeline);

        PipelineInstanceQuery pipelineInstanceQuery = new PipelineInstanceQuery();
        pipelineInstanceQuery.setState(PipelineFinal.RUN_RUN);
        pipelineInstanceQuery.setPipelineId(pipelineId);
        List<PipelineInstance> pipelineInstanceList = pipelineInstanceService.findPipelineInstanceList(pipelineInstanceQuery);
        for (PipelineInstance pipelineInstance : pipelineInstanceList) {
            pipelineInstance.setRunStatus(PipelineFinal.RUN_HALT);
            int runtime = pipelineInstanceService.findInstanceRuntime(pipelineInstance.getInstanceId());
            pipelineInstance.setRunTime(runtime);
            pipelineInstanceService.updateInstance(pipelineInstance);
        }
        removeExecCache(pipelineId);
    }

    public void removeExecCache(String pipelineId) {
        String instanceId = pipelineIdOrInstanceId.get(pipelineId);
        PipelineInstanceServiceImpl.runTimeMap.remove(instanceId);
        pipelineInstanceService.stopThread(instanceId);
        pipelineIdOrInstanceId.remove(pipelineId);
    }

    @Override
    public void keepOn(String pipelineId) {
        Agent agent = pipelineIdOrAgentId.get(pipelineId);
        String id = agent.getAddress();
        AgentMessage agentMessage = new AgentMessage();
        agentMessage.setType("keepOn");
        agentMessage.setMessage(pipelineId);
        agentMessage.setPipelineId(pipelineId);
        WebSocketSession session = SocketServerHandler.sessionMap.get(id);
        if (Objects.isNull(session)) {
            throw new SystemException("客户端推送消息失败，无法获取客户端连接,客户端信息：" + id);
        }

        try {
            SocketServerHandler.instance().sendHandleMessage(id, agentMessage);
        } catch (Exception e) {
            throw new SystemException("客户端推送消息失败,错误信息：" + e.getMessage());
        }
    }

}


































